//Chip type           : AT90S2313
//Clock frequency     : 11.059000 MHz
//Memory model        : Tiny
//External SRAM size  : 0
//Data Stack size     : 32
//*********************************************/
/*Algorithm:
' Interrupt on both edges of phase A
' Test phase B in ISR
' When phase B <> phase A then decrement counter
' When phase B = phase A then increment counter
' Convert counter value to displacment units
' Format value in string and display
'
'-------------------------------------------------------------------------------
'
' Setup for AT90S2313-10
'
' INT0 edge is set to rising edge
' INT1 edge is set to falling edge
' Encoder phase A to INT0 and INT1  (PD2 and PD3)
' Encoder phase B to PD4
' Pushbutton between Portd.5 and GND  (display zero)
' LCD at PortB
*/
#include <90s2313.h>
#define TRUE 1
#define FALSE 0
#define Phase_a PIND.2                   // 'INT0 also connected to Pind.3 = INT1
#define Phase_b PIND.4



bit  Rem_phase_a ;  //current encoder state memory
bit  Rem_phase_b ;  //current encoder state memory
signed long int encoder_counter1;
unsigned char command, count;

/* union declaration */
union alpha {
            unsigned char byte[4];
            signed long int encoder_counter2 ;
            } data;
/////////////////////////////////////////////////////////////////////

void Calculate_step()
{
//test level of A and B phases again to protect chip from accidental noise
if ((Rem_phase_a != Phase_a || Rem_phase_b != Phase_b) && (Rem_phase_a !=  Phase_a || Rem_phase_b != Phase_b)) // O.K, something really changed in levels 
  {
	if (Phase_a == Phase_b) encoder_counter1++;
	else encoder_counter1--;

   Rem_phase_a =  Phase_a; // remember current state
   Rem_phase_b =  Phase_b;   // remember current state
  }

}

///////////////////////////////////////////////////////////////////
// External Interrupt 0 service routine
interrupt [EXT_INT0] void ext_int0_isr(void)
{
// Place your code here
Calculate_step();
}

///////////////////////////////////////////////////////////////////
// External Interrupt 1 service routine
interrupt [EXT_INT1] void ext_int1_isr(void)
{
// Place your code here
Calculate_step();
}

////////////////////////////////////////////////////////////////////
// Declare your global variables here
void main(void)
{

count = 0;
encoder_counter1 = 0;
data.encoder_counter2 = 0;
Rem_phase_a =  Phase_a; // initialization: remember current state
Rem_phase_b =Phase_b;   // initialization: remember current state

// Declare your local variables here

// Input/Output Ports initialization
// Port B initialization
// Func0=Out Func1=Out Func2=Out Func3=Out Func4=Out Func5=Out Func6=Out Func7=Out 
// State0=1 State1=1 State2=1 State3=1 State4=1 State5=1 State6=1 State7=1 
PORTB=0xFF;
DDRB=0xFF;


// Port D initialization
// Func0=In Func1=Out Func2=In Func3=In Func4=In Func5=In Func6=In 
// State0=T State1=1 State2=T State3=T State4=T State5=T State6=T 
PORTD=0x02;
DDRD=0x02;


// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: Timer 0 Stopped
TCCR0=0x00;
TCNT0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1 output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
OCR1H=0x00;
OCR1L=0x00;

// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Rising Edge
// INT1: On
// INT1 Mode: Falling Edge
GIMSK=0xC0;
MCUCR=0x0B;
GIFR=0xC0;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x00;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
// Analog Comparator Output: Off
ACSR=0x80;
PORTD.1 = 1;
// Global enable interrupts
#asm("sei")

 while (1)  // Place your code here
 {
      if (PIND.0 == 0 && PORTD.1 == 1 )   // sihronization of transfer
      {      
        command = 0;    // reset command
        //get command 00 - reset counter, 01 - read lowest byte, 10 - read next byte, which is pointed by counter, 11 - do nothing
        if (PIND.5 == 0) command++;
        if (PIND.6 == 0) {command++;  command++; }  
       //Now command = 3 - reset counter, 1 - send lowest byte, 2 - send second byte, which is pointed by counter, 0 - do nothing
      
        if (command == 1) //1 - send lowest byte, and set count pointer to next byte
        {
        #asm("cli"); // disable interrupts
        data.encoder_counter2 = encoder_counter1; //copy data to buffer and send lowest byte to Port B
        #asm("sei"); //enable interrupts
        PORTB = data.byte[0]; 
        PORTD.1 = 0; // data are on 
        count = 1;           
        } 
        
        else if (command == 2) 
        {        
              if (count == 0)  
               { 
                #asm("cli"); // disable interrupts
        	data.encoder_counter2 = encoder_counter1; //copy data to buffer 
        	#asm("sei"); //enable interrupts
               }
          PORTB = data.byte[count]; //2 - send next byte      
          PORTD.1 = 0; // data are on PORTB ready   
          count++;          
          if (count > 3) count = 0; //there is no more to read, reset count
        
        }
        else if (command == 3) //3 - reset counter
        {
        count = 0;
        encoder_counter1 = 0;         
        PORTB = 0xff;
        PORTD.1 = 0;
        }
        else PORTD.1 = 0; //do nothing, just respond
                         
      } //end of  sihronization of transfer
      else if (PIND.0 == 1)  {PORTB = 0xff; PORTD.1 = 1; } // end of transfer

  } //end while(1)
} //end main
